@@ -1,3 +1,3 @@
 from .features.extractor import FeatureExtractor
-from .features.generator import EventGenerator
 from .models import Recommender
+from .schema import Schema
